<?php
/**
 * Joomla! component SimpleLists
 *
 * @author Yireo
 * @package SimpleLists
 * @copyright Copyright (C) 2011
 * @license GNU Public License
 * @link http://www.yireo.com/
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

// Require the YireoHelper class
require_once JPATH_ADMINISTRATOR.DS.'components'.DS.'com_simplelists'.DS.'lib'.DS.'helper.php';

// Load the plugin helper        
jimport('joomla.plugin.helper');

/**
 * Simplelists Plugin Helper
 * 
 * @package Joomla
 * @subpackage Simplelists
 */
class SimplelistsPluginHelper
{
    /**
     * Method to load a specific plugin
     */
    public function getPlugin($type = null, $name = null)
    {
        // @todo: Does not work in J1.7
        $plugin = JPluginHelper::getPlugin($type, $name);
        if(empty($plugin) || !is_object($plugin)) {
            return null;
        }

        if(YireoHelper::isJoomla15()) {
        	$path = JPATH_PLUGINS.DS.$plugin->type.DS.$plugin->name.'.php';
        } else {
        	$path = JPATH_PLUGINS.DS.$plugin->type.DS.$plugin->name.DS.$plugin->name.'.php';
        }

        if(!file_exists($path)) {
            return null;
        }

        // Include the plugin-file
        require_once($path);

        // Determine the class-name and return an instance
        $class = 'plg'.$plugin->type.$plugin->name;
        if(class_exists($class)) {
		    $dispatcher =& JDispatcher::getInstance();
            $plugin = new $class($dispatcher, (array)$plugin);
            return $plugin;
        }

        return null;
    }

    /**
     * Method to load plugins of a specific type
     */
    public function getPlugins($type = null)
    {
        // Load the plugins
        $plugins = JPluginHelper::getPlugin($type);
        foreach($plugins as $index => $plugin) {
            $plugin = self::getPlugin($plugin->type, $plugin->name);
            if($plugin == null) {
                unset($plugins[$index]); 
            } else {
                $plugins[$index] = $plugin;
            }
        }

        return $plugins;
    }

    /**
     * Method to return the title of a specific plugin
     */
    public function getPluginLinkTitle($item)
    {
        $plugin = self::getPlugin('simplelistslink', $item->link_type);
        if(!empty($plugin)) {
            return $plugin->getTitle();
        }
    }

    /**
     * Method to return the link-name of a specific plugin
     */
    public function getPluginLinkName($item)
    {
        $plugin = self::getPlugin('simplelistslink', $item->link_type);
        if(!empty($plugin)) {
            return $plugin->getName($item->link);
        }
    }

    /**
     * Method to return the hidden-value of a specific plugin
     */
    public function getPluginLinkHidden($item)
    {
        $plugin = self::getPlugin('simplelistslink', $item->link_type);
        if(!empty($plugin)) {
            return $plugin->getHidden($item);
        }
    }

    /**
     * Method to return the URL-value of a specific plugin
     */
    public function getPluginLinkUrl($item)
    {
        $item->params = YireoHelper::toParameter($item->params);
        $return = null;

        if($item->params->get('link_show', 1) == 0 && !empty($item->link)) {
            $url = 'index.php?option=com_simplelists&view=item&task=hidden&tmpl=component&id='.$item->id;
            if($item->alias) $url .= ':'.$item->alias;
            $return = JRoute::_( $url );

        } else {
            $plugin = self::getPlugin('simplelistslink', $item->link_type);
            if(!empty($plugin)) {
                $return = $plugin->getUrl($item);
            }
        }
        return $return;
    }
}
